//Created by Steve Cooper
//www.gadgetshed.co.uk
//Email: gadgetshed@gmail.com
//Version 1.1 - 16 Mar 2021 Fully parameterised

//BOX parameters
    ang=15;                 // angle of slope
    bl=270;                 // box length
    bh=80;                  // box height at rear - needs to be large enough to accomodate switches at front edge
    bw=200;                 // box width

    //Tweaks to border and facedepth
        facedepth=3;            // face depth
        border=10;               //offset of border - top and bottomn lip for base is border-border_outer
        border_outer=5;         //outer case border

//Board mount pararmeters
    bm_required=1;          //set to 0 if not required
    bm_xoffset=40;          //mount x offset in case 
    bm_yoffset=90;          //mount y offset in case

    //Below parameters set for Pi Zero sizing
        bm_xdia=58;             //distance between board mount holes - set for pi zero
        bm_ydia=23;             //set for pi zero
        bm_h=6;                 //mount point height from face
        bm_o=3.2;               //mount point outer size
        bm_i=1.2;               //mount point inner size

//screen cutout parameters 
    scr_required=1;         //set to 0 if not required
    scr_x_offset=bl/2;      //offset to centre of screen on x axis
    scr_y_offset=bw/2;      //offset to centre of screen on y axis
    
    //ONLY Change below to change cutout tool shape - set for SSD1306
        scr_w=22;               //display area width
        scr_h=11;               //display area height
        scr_glass_w=26;         //width of glass panel
        scr_glass_h=17.5;       //height of glass panel
        scr_glass_offset=1.5;   //space between top of screen and top of glass
        scr_pins_w=14;          //width of area that has pin headers
        scr_pins_h=28;          //height of screen pcb
        scr_pins_offset=6;      //space between top of screen and top of pcb
        scr_recess=1.5;         //depth of cut before recess cut
        scr_cut_height=8;       //height of cutting piece

//power hole
    ph_required=1;          //setup 0 if not required
    ph_squareformat=1;      //Either Square=1 or Round=0
    ph_w=11.25;             //width of power socket if square or diameter if round
    ph_h=8;                 //height of power socket (not used for round
    ph_offset=1;            //percentage offset from side 0 on left -> 1 full on right  side must be between 0 and 1
    ph_buildsupport=1;      //build a support for power connector

//Button parameters
    font = "Arial:style=Bold";
    textdepth = 2.0;        //must be less than facedepth
    fontsize = 10;           //fontsize for text
    ledsize = 5.2;          //diameter of led hole
    buttonsize = 12.4;      //diameter of hole for footswitch
    
    /*  The button map controls where button go each button has 8 parameters
        You can add or remove extra buttons as required 
        just remember to put a comma at the end of each previous new line
        
        Positions are from top left corner with x going across the box and Y going down
            0.  X position on face - best to do as ratio of box length (bl)
            1.  Y position on face - best to do as ratio of box width (bw)
            2.  Any text required leave blank for no text
            3.  Text X offset from button
            4.  Text Y offset from button
            5.  LED indicator - set to 1 if led hole required or 0 if no led
            6.  LED X offset from button
            7.  LED Y offset from button
    */
    
    buttonmap = [
                    [(bl/12)*2,(bw/4)*3,"Drive",0,-20,1,0,20],
                    [(bl/12)*6,(bw/4)*3,"Mod",0,-20,1,0,20],
                    [(bl/12)*10,(bw/4)*3,"Delay",0,-20,1,0,20],
                    [(bl/12)*2,(bw/4)*1,"Up",0,20,0,0,20],
                    [(bl/12)*6,(bw/4)*1,"Down",0,20,0,0,20],
                    [(bl/12)*10,(bw/4)*1,"Select",0,20,0,0,20]
                ];

//General parameters
    $fn=120;                        //circle accurancy
    tolerance =0.4;                 // tolerance for making base plate smaller than cutout
    offset=facedepth/sin(90-ang);   // used to set depth of cover - DO NOT CHANGE
        
/*********************************************************************************
***************************** START of processing ********************************
*********************************************************************************/

main();  //Make main unit
plate(); //Make base plate


/*********************************************************************************
***************************** END of processing **********************************
*********************************************************************************/

//********************************** MODULES ***************************************

//Create the main box
module main(){
    difference(){
        union(){
            mainbox();
            if (bm_required==1) pillars();
        }
        //screen recess
        if (scr_required==1) translate([scr_x_offset,scr_y_offset,0])screen();
        button();
    }
}

//Create base plate
module plate(){
    translate([0,border_outer -bw-10,0])
    difference(){
            translate([border_outer,border_outer,-.01])linear_extrude(height=3.01) square([bl-border_outer*2-tolerance*2,bw-border_outer*2-tolerance*2]);
            translate([(border_outer+5)-tolerance,(border_outer+5)-tolerance,-0.1])cylinder(r=1.5,h=12);
            translate([(border_outer+5)-tolerance,bw-(border_outer+5)-tolerance,-0.1])cylinder(r=1.5,h=12);
            translate([bl-(border_outer+5)-tolerance,(border_outer+5)-tolerance,-0.1])cylinder(r=1.5,h=12);
            translate([bl-(border_outer+5)-tolerance,bw-(border_outer+5)-tolerance,-0.1])cylinder(r=1.3,h=12);
            translate([(border_outer+5)-tolerance,(border_outer+5)-tolerance,1.01])cylinder(r=3,h=2);
            translate([(border_outer+5)-tolerance,bw-(border_outer+5)-tolerance,1.01])cylinder(r=3,h=2);
            translate([bl-(border_outer+5)-tolerance,(border_outer+5)-tolerance,1.01])cylinder(r=3,h=2);
            translate([bl-(border_outer+5)-tolerance,bw-(border_outer+5)-tolerance,1.01])cylinder(r=3,h=2);     
    }
}

//Create screen cutout
module screen(){
    //create a cut for the screen module
    translate([0,0,-0.01])linear_extrude(height=scr_cut_height+0.01) square([scr_w,scr_h],center=true);
    translate([0,+scr_pins_h/2-scr_pins_offset-scr_h/2,scr_recess])linear_extrude(height=scr_cut_height-scr_recess) square([scr_pins_w,scr_pins_h],center=true);
    translate([0,scr_glass_h/2-scr_glass_offset-scr_h/2,scr_recess])linear_extrude(height=scr_cut_height-scr_recess) square([scr_glass_w,scr_glass_h],center=true);
}

//Create pcb mounts
module pillars(){
    //board mount pillars drop in by 0..1 to ensure render into face
    translate([bm_xoffset,bm_yoffset,offset-0.1])pillar(bm_h,bm_o,bm_i);
    translate([bm_xoffset,bm_yoffset+bm_ydia,offset-0.1])pillar(bm_h,bm_o,bm_i);
    translate([bm_xoffset+bm_xdia,bm_yoffset,offset-0.1])pillar(bm_h,bm_o,bm_i);
    translate([bm_xoffset+bm_xdia,bm_yoffset+bm_ydia,offset-0.1])pillar(bm_h,bm_o,bm_i);
}

//Create an idividual mount point
module pillar(ph,pr,pi){
    difference(){
        cylinder(r=pr,h=ph);
        cylinder(r=pi,h=ph+0.1);
    }
}

//Create main box
module mainbox()
{
    difference()
    {
        rotate([180-ang,0,0])
        translate([0,-bw,-bh]) 
        difference()
        {
            box(bh,bw,bl,ang);
            //inner cut part
            translate([border_outer+10,border,-.01])inner_cutout(bh-offset,bw-border*2,bl-border_outer*2-20,ang);
            //base
            translate([(border_outer),(border_outer),-.01])linear_extrude(height=3.01) square([bl-(border_outer)*2,bw-(border_outer)*2]);
            //screwholes
            translate([(border_outer+5),(border_outer+5),-0.01])cylinder(r=1.5,h=25);
            translate([(border_outer+5),bw-(border_outer+5),-0.01])cylinder(r=1.5,h=25);
            translate([bl-(border_outer+5),(border_outer+5),-0.01])cylinder(r=1.5,h=25);
            translate([bl-(border_outer+5),bw-(border_outer+5),-0.01])cylinder(r=1.5,h=25);
            //powersocket hole
            if (ph_required==1&&ph_squareformat==1)
            translate([border_outer+10+((bl-border_outer*2-20)-(ph_w+4))*ph_offset+2,bw-35.9,2.99])linear_extrude(height=ph_h) square([ph_w,36]);
            if (ph_required==1&&ph_squareformat==0)
            translate([border_outer+10+((bl-border_outer*2-20)-(ph_w+4))*ph_offset+2,bw-35.9,4])translate([ph_w/2,35.99,ph_w/2])rotate([90,0,0])linear_extrude(height=36) circle(r=ph_w/2);
        }
    }
}

//Create outer shell
module box(ph,pw,pl,pa){
    translate([0,pw,ph])
    rotate([180,0,0])
    intersection()
    {
        rotate([pa,0,0])linear_extrude(height=ph) square([pl,pw*2]);
        linear_extrude(height=ph) square([pl,pw]);
    }
}

//Create initial cutout of outer shell
module inner_cutout(ph,pw,pl,pa){
    translate([0,pw+border,ph])
    rotate([180,0,0])
    intersection()
    {
        rotate([pa,0,0])linear_extrude(height=ph) square([pl,pw*2]);
        difference(){
            translate([0,border,0])linear_extrude(height=ph) square([pl,pw]);
            //power scoket support
            if (ph_required==1 && ph_buildsupport==1)
            translate([(pl-(ph_w+4))*ph_offset,-0.01,-0.01])linear_extrude(height=ph+0.02) square([ph_w+4,27.01]);
        }
    }
}

module button()
{
        for (a = [0:len(buttonmap)-1])
        {
            //Button hole
            translate([buttonmap[a][0],buttonmap[a][1],-0.01])cylinder(r=buttonsize/2,h=facedepth*2);
            //led hole
            if(buttonmap[a][5]==1)
                translate([buttonmap[a][0]+buttonmap[a][6],buttonmap[a][1]+buttonmap[a][7],-0.01])cylinder(r=ledsize/2,h=facedepth*2);
            //text
            if(buttonmap[a][2]!="")
                translate([buttonmap[a][0]+buttonmap[a][3],buttonmap[a][1]+buttonmap[a][4],textdepth])
                rotate([0,180,180])
                linear_extrude(height = textdepth+0.01) 
                text(text = str(buttonmap[a][2]), font = font, size = fontsize, halign="center", valign="center");
        }
}    
